
import uuid

from django.conf import settings
from django.db import models
from django.db.models import F
from django.contrib.auth.models import User, Group
from django.utils.translation import gettext_lazy as _
from django.utils import timezone
from django.db.models.functions import Lower

from uhepp_api.models import Token

PRIVATE_LEVEL = 10
INTERNAL_LEVEL = 20
PUBLIC_LEVEL = 30
VISIBILITY_LEVELS = [
    (PRIVATE_LEVEL, _("Accessible only by you")),
    (INTERNAL_LEVEL, _("Accessible only by members of your collaboration")),
    (PUBLIC_LEVEL, _("Publicly accessible")),
]

class Collection(models.Model):
    title = models.CharField(_("Collection title"), max_length=100)
    slug = models.SlugField(_("Collection URL slug"))
    owner = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        related_name='collections'
    )
    description = models.TextField(
        _("Collection description"),
        null=True,
        blank=True
    )
    visibility = models.IntegerField(
        _("Collection visibility level"),
        choices=VISIBILITY_LEVELS,
        default=PRIVATE_LEVEL,
    )
    created = models.DateTimeField(auto_now_add=True)
    activity = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ['-activity', Lower('title')]

    def __str__(self):
        return f"{self.owner}: {self.title}"

class LazyPlotManger(models.Manager):
    use_for_related_fields = True

    def get_queryset(self, *args, **kwargs):
        return super() \
            .get_queryset(*args, **kwargs) \
            .defer("uhepp") \
            .order_by('-uploaded', 'title')

class Plot(models.Model):
    """Database record of a uhepp compliant plot"""
    uuid = models.UUIDField(primary_key=True, default=uuid.uuid4)
    uhepp = models.JSONField()
    ui_view_count = models.IntegerField(default=0)
    api_view_count = models.IntegerField(default=0)
    title = models.CharField(max_length=255, null=True, blank=True)
    uploaded = models.DateTimeField(auto_now_add=True)
    comment = models.TextField(null=True, blank=True)

    collection = models.ForeignKey(
        Collection,
        on_delete=models.CASCADE,
        related_name='plots'
    )

    objects = LazyPlotManger()

    def view_count(self):
        # Plus 1 since increments are not updated
        return self.api_view_count + 1

    def increment_api_view_count(self):
        """Increment the view counter without a race condition"""
        Plot.objects.filter(uuid=self.uuid).update(
            api_view_count=F('api_view_count') + 1
        )

    def increment_ui_view_count(self):
        """Increment the view counter without a race condition"""
        Plot.objects.filter(uuid=self.uuid).update(
            ui_view_count=F('ui_view_count') + 1
        )

    def save(self, *args, **kwds):
        if self.title is None:
            self.title = str(self)

        if self.collection:
            self.collection.activity = timezone.now()
            self.collection.save()

        return super().save(*args, **kwds)

    def __str__(self):
        if self.title:
            return self.title

        try:
            metadata = self.uhepp.get("metadata", {})
            filename = metadata.get("filename")
            return filename or str(self.uuid)
        except (AttributeError, KeyError):
            return str(self.uuid)

class Profile(models.Model):
    """User profile"""
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        primary_key=True
    )
    cern_upn = models.CharField(
        _("User Principal Name"),
        max_length=100,
        unique=True
    )
    home_institute = models.CharField(
        _("Home Institute"),
        max_length=100,
        null=True,
        blank=True
    )
    cern_person_id = models.IntegerField(
        _("CERN Person ID"),
        null=True,
        blank=True
    )
    visibility = models.IntegerField(
        _("Profile visibility level"),
        choices=VISIBILITY_LEVELS,
        default=PRIVATE_LEVEL,
    )

    def __str__(self):
        return self.cern_upn
